package com.tzld.piaoquan.recommend.server.service.filter.strategy;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.TypeReference;
import com.ctrip.framework.apollo.spring.annotation.ApolloJsonValue;
import com.tzld.piaoquan.recommend.server.service.filter.FilterParam;
import com.tzld.piaoquan.recommend.server.service.filter.FilterStrategy;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Slf4j
@Component
public class VovLowerStrategy implements FilterStrategy {

    @ApolloJsonValue("${vov.filter.condition:{}}")
    private Map<String, Double> vovFilterCondition;

    @Autowired
    @Qualifier("redisTemplate")
    private RedisTemplate<String, String> redisTemplate;

    private static final String KEY_FORMAT = "redis:vid_vov_daily4filter:%d";

    @Override
    public List<Long> filter(FilterParam param) {

        List<Long> result = new ArrayList<>();

        List<Long> videoIds = param.getVideoIds();
        if (CollectionUtils.isEmpty(videoIds)) {
            log.info("VOV过滤 -- videoIds为空，跳过");
            return result;
        }

        List<String> redisKeys = videoIds.stream()
                .map(r -> String.format(KEY_FORMAT, r))
                .collect(Collectors.toList());
        List<String> vovInfos = redisTemplate.opsForValue().multiGet(redisKeys);
        if (CollectionUtils.isEmpty(vovInfos) || vovInfos.size() != videoIds.size()) {
            log.info("VOV过滤 -- 获取到的视频VOV信息为空或长度与videoIds长度不一致，跳过: {}, {}", videoIds.size(), vovInfos.size());
            return result;
        }

        List<Long> removeIds = new ArrayList<>();

        for (int i = 0; i < videoIds.size(); i++) {
            Long videoId = videoIds.get(i);
            String vovInfo = vovInfos.get(i);
            if (isShouldFilter(vovInfo)) {
                removeIds.add(videoId);
                continue;
            }

            result.add(videoId);
        }

        log.info("VOV -- 本次召回的ID列表为: {}, 被过滤掉的ID列表为: {}, 剩下的ID列表为: {}", videoIds, removeIds, result);

        return result;
    }


    private boolean isShouldFilter(String vovInfo) {
        try {

            if (StringUtils.isEmpty(vovInfo)) {
                return false;
            }

            Map<String, Double> vovInfoMap = JSON.parseObject(vovInfo, new TypeReference<Map<String, Double>>() {
            });
            if (MapUtils.isEmpty(vovInfoMap)) {
                return false;
            }

            if (isNewVideo(vovInfoMap) || isUptrendVideo(vovInfoMap)) {
                return false;
            }

            // 预估VOV
            double t1Vov0 = vovInfoMap.getOrDefault("1_vov0", 0d);
            double t2Vov0 = vovInfoMap.getOrDefault("2_vov0", 0d);

            double estimated2Vov0Alpha = vovFilterCondition.getOrDefault("estimated_2_vov_0_alpha", 0.2d);
            double estimated1Vov0Alpha = vovFilterCondition.getOrDefault("estimated_1_vov_0_alpha", 0.8d);
            double estimatedVovThreshold = vovFilterCondition.getOrDefault("estimated_vov_threshold", 0.19d);

            double estimatedVov = t1Vov0;
            if (t2Vov0 != 0) {
                estimatedVov = estimated2Vov0Alpha * t2Vov0 + estimated1Vov0Alpha * t1Vov0;
            }

            return estimatedVov < estimatedVovThreshold;
        } catch (Exception e) {
            log.info("VOV过滤 -- 异常: ", e);
        }
        return false;
    }

    // 三天曝光都小于阈值的 认定为新视频
    private boolean isNewVideo(Map<String, Double> vovInfoMap) {
        double t2ViewPvCondition = vovFilterCondition.getOrDefault("t_2_view_pv", 50d);
        double t3ViewPvCondition = vovFilterCondition.getOrDefault("t_3_view_pv", 50d);
        double t4ViewPvCondition = vovFilterCondition.getOrDefault("t_4_view_pv", 50d);
        double t2ViewPv = vovInfoMap.getOrDefault("2_vov0_分母", 0d);
        double t3ViewPv = vovInfoMap.getOrDefault("3_vov0_分母", 0d);
        double t4ViewPv = vovInfoMap.getOrDefault("4_vov0_分母", 0d);

        return t2ViewPv < t2ViewPvCondition && t3ViewPv < t3ViewPvCondition && t2ViewPv < t4ViewPvCondition;
    }

    // 最近两天的vov满足并值，且昨天的vov-前天的vov 达到一定的涨幅认定为上升趋势视频
    private boolean isUptrendVideo(Map<String, Double> vovInfoMap) {

        double t1VovCondition = vovFilterCondition.getOrDefault("t_1_vov", 0.2d);
        double t2VovCondition = vovFilterCondition.getOrDefault("t_2_vov", 0d);
        double vovDiff = vovFilterCondition.getOrDefault("12_vov_diff", 0.1);

        double t1Vov0 = vovInfoMap.getOrDefault("1_vov0", 0d);
        double t2Vov0 = vovInfoMap.getOrDefault("2_vov0", 0d);
        double t12VovDiff = t1Vov0 - t2Vov0;

        return t2Vov0 > t2VovCondition && t1Vov0 > t1VovCondition && t12VovDiff > vovDiff;

    }

}
